import re


STRING_VALS = (
    (
        'aws_secret_key',
        'Amazon Web Services secret key',
        (
            re.compile(r'(\'|")[A-Za-z0-9\\\+]{40}(\'|")'),
            re.compile(r'(\b|_)AWS(\b|_)', re.IGNORECASE)
        ),
        all
    ),
)

LINE_VALS = (
    (
        'diff',
        'Possible SCM diff in code',
        (
            re.compile(r'^<<<<<<< .*$'),
            re.compile(r'^>>>>>>> .*$')
        ),
    ),
    (
        'ssh_rsa_private_key',
        'Possible SSH private key',
        re.compile(r'^-{5}(BEGIN|END)\s+RSA\s+PRIVATE\s+KEY-{5}$')
    ),
    (
        'ssh_rsa_public_key',
        'Possible SSH public key',
        re.compile('^ssh-rsa\s+AAAA[0-9A-Za-z+/]+[=]{0,3}\s*([^@]+@[^@]+)?$')
    ),
)

VAR_NAMES = (
    (
        'password',
        'Possible hardcoded password',
        re.compile(r'(\b|[A-Z0-9_]*_)PASSWORD(_[A-Z0-9_]*|\b)\s*=\s(\'|")[^\'"]+(\'|")')
    ),
    (
        'secret',
        'Possible hardcoded secret key',
        re.compile(r'(\b|[A-Z0-9_]*_)SECRET(_[A-Z0-9_]*|\b)\s*=\s(\'|")[^\'"]+(\'|")')
    ),
)


def check_line(line, check_list):
    messages = []

    for tup in check_list:
        if len(tup) == 3:
            key, msg, regexps = tup
            cond = any
        else:
            key, msg, regexps, cond = tup

        if not isinstance(regexps, (list, tuple)):
            regexps = [regexps]
        if cond([regexp.search(line) for regexp in regexps]):
            messages.append((key, msg))

    return messages


def check_file(filepath):
    with open(filepath) as to_check:
        return check_file_contents(to_check.read())


def check_file_contents(file_contents):
    messages = []

    for line_number0, line in enumerate(file_contents.split('\n')):
        for check_list in (STRING_VALS, LINE_VALS, VAR_NAMES):
            messages += [(line_number0+1, key, msg) for key, msg in check_line(line, check_list)]

    return messages



# -*- coding: UTF-8 -*-
from distutils.core import setup
from setuptools import find_packages
from dodgy import __pkginfo__

_packages = find_packages(exclude=["*.tests", "*.tests.*", "tests.*", "tests"])

_short_description = "Dodgy: Searches for dodgy looking lines in Python code"

_install_requires = [
    'argparse',
]

_classifiers = (
    'Development Status :: 2 - Pre-Alpha',
    'Environment :: Console',
    'Intended Audience :: Developers',
    'License :: OSI Approved :: MIT License',
    'Operating System :: Unix',
    'Topic :: Software Development :: Quality Assurance',
    'Programming Language :: Python :: 2.6',
    'Programming Language :: Python :: 2.7',
)

setup(
    name='dodgy',
    url='https://github.com/landscapeio/dodgy',
    author='landscape.io',
    author_email='code@landscape.io',
    description=_short_description,
    install_requires=_install_requires,
    scripts=['bin/dodgy'],
    version=__pkginfo__.get_version(),
    packages=_packages,
    license='MIT',
    keywords='check for suspicious code',
    classifiers=_classifiers
)



import sys
import re
import os
import mimetypes
import json
from dodgy.checks import check_file


IGNORE_PATHS = [re.compile(patt % {'sep': re.escape(os.path.sep)}) for patt in (
    r'(^|%(sep)s)\.[^\.]',   # ignores any files or directories starting with '.'
    r'^tests?%(sep)s?',
    r'%(sep)stests?(%(sep)s|$)',
)]


def list_files(start_path):
    filepaths = []
    for root, _, files in os.walk(start_path):
        for file_name in files:
            filepaths.append(os.path.join(root, file_name))
    return filepaths


def run_checks(directory, ignore_paths=None):
    warnings = []

    ignore_paths = ignore_paths or []
    ignore_paths = [re.compile(patt) for patt in ignore_paths]
    ignore_paths += IGNORE_PATHS

    filepaths = list_files(directory)
    for filepath in filepaths:
        relpath = os.path.relpath(filepath, directory)
        if any([ignore.search(relpath) for ignore in ignore_paths]):
            continue

        # this is a naive check to skip binary files, it's probably okay for now
        mimetype = mimetypes.guess_type(filepath)
        if mimetype[0] is None or not mimetype[0].startswith('text/'):
            continue

        for msg_parts in check_file(filepath):
            warnings.append({
                'path': relpath,
                'line': msg_parts[0],
                'code': msg_parts[1],
                'message': msg_parts[2]
            })

    return warnings


def run():
    warnings = run_checks(os.getcwd())
    output = json.dumps({'warnings': warnings}, indent=2)
    sys.stdout.write(output + '\n')


if __name__ == '__main__':
    run()
